<?php
	function gregorianToJulian(int $year, int $month, int $day){
	        if ($month < 3)
	        {
	            $year -= 1;
	            $month += 12;
	        }

	        $a = floor($year / 100.0);
	        $b = ($year === 1582 && ($month > 10 || ($month === 10 && $day > 4)) ? -10 :
	            ($year === 1582 && $month === 10 ? 0 :
	                ($year < 1583 ? 0 : 2 - $a + floor($a / 4.0))));

	        return floor(365.25 * ($year + 4716)) + floor(30.6001 * ($month + 1)) + $day + $b - 1524;
	}
	
	function hijriToJulian(int $year, int $month, int $day){
	        return floor((11 * $year + 3) / 30) + floor(354 * $year) + floor(30 * $month)
	            - floor(($month - 1) / 2) + $day + 1948440 - 386;
	}
	
	function julianToGregorian(float $julianDay){
        $b = 0;
        if ($julianDay > 2299160)
        {
            $a = floor(($julianDay - 1867216.25) / 36524.25);
            $b = 1 + $a - floor($a / 4.0);
        }

        $bb = $julianDay + $b + 1524;
        $cc = floor(($bb - 122.1) / 365.25);
        $dd = floor(365.25 * $cc);
        $ee = floor(($bb - $dd) / 30.6001);

        $day = ($bb - $dd) - floor(30.6001 * $ee);
        $month = $ee - 1;

        if ($ee > 13)
        {
            $cc += 1;
            $month = $ee - 13;
        }

        $year = $cc - 4716;

        return (object) ['year' => (int) $year, 'month' => (int) $month, 'day' => (int) $day];
    }
    
	function julianToHijri(float $julianDay){
        $y = 10631.0 / 30.0;
        $epochAstro = 1948084;
        $shift1 = 8.01 / 60.0;

        $z = $julianDay - $epochAstro;
        $cyc = floor($z / 10631.0);
        $z = $z - 10631 * $cyc;
        $j = floor(($z - $shift1) / $y);
        $z = $z - floor($j * $y + $shift1);

        $year = 30 * $cyc + $j;
        $month = (int)floor(($z + 28.5001) / 29.5);
        if ($month === 13)
        {
            $month = 12;
        }

        $day = $z - floor(29.5001 * $month - 29);

        return (object) ['year' => (int) $year, 'month' => (int) $month, 'day' => (int) $day];
    }
    /*
function intPart($float){
    if ($float < -0.0000001)
        return ceil($float - 0.0000001);
    else
        return floor($float + 0.0000001);
}
function Greg2Hijri($day, $month, $year, $string = false){
    $day   = (int) $day;
    $month = (int) $month;
    $year  = (int) $year;

    if (($year > 1582) or (($year == 1582) and ($month > 10)) or (($year == 1582) and ($month == 10) and ($day > 14))){
        $jd = intPart((1461*($year+4800+intPart(($month-14)/12)))/4)+intPart((367*($month-2-12*(intPart(($month-14)/12))))/12)-
        intPart( (3* (intPart(  ($year+4900+    intPart( ($month-14)/12)     )/100)    )   ) /4)+$day-32075;
    }else{
        $jd = 367*$year-intPart((7*($year+5001+intPart(($month-9)/7)))/4)+intPart((275*$month)/9)+$day+1729777;
    }
    $l = $jd-1948440+10632;
    $n = intPart(($l-1)/10631);
    $l = $l-10631*$n+354;
    $j = (intPart((10985-$l)/5316))*(intPart((50*$l)/17719))+(intPart($l/5670))*(intPart((43*$l)/15238));
    $l = $l-(intPart((30-$j)/15))*(intPart((17719*$j)/50))-(intPart($j/16))*(intPart((15238*$j)/43))+29;
    
    $month = intPart((24*$l)/709);
    $day   = $l-intPart((709*$month)/24);
    $year  = 30*$n+$j-30;
    
    $date = array();
    $date['year']  = $year;
    $date['month'] = $month;
    $date['day']   = $day;
    if (!$string)
        return $date;
    else
        return "{$year}-{$month}-{$day}";
}*/
	$hijriMonth = array ("محرم", "صفر", "ربیع الاول", "ربیع الثانی", "جمادی اولی", "جمادی الثانی", "رجب", "شعبان", "رمضان", "شوال", "ذوالقعده", "ذوالحجه");
//------------------------------------------------------------------------------------------------------------------------------
	function GMonthName($month){
	    if($month=="01") return "ژانویه";
	    if($month=="02") return "فوریه";
	    if($month=="03") return "مارس";
	    if($month=="04") return  "آپریل";
	    if($month=="05") return "می";
	    if($month=="06") return "جوئن";
	    if($month=="07") return "جولای";
	    if($month=="08") return "آگوست";
	    if($month=="09") return "سپتامبر";
	    if($month=="10") return "اکتبر";
	    if($month=="11") return "نوامبر";
	    if($month=="12") return "دسامبر";
	}

	function convert_to_en_number($string) {
	    $persian = ['۰', '۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹'];
	    $arabic = ['٩', '٨', '٧', '٦', '٥', '٤', '٣', '٢', '١','٠'];

	    $num = range(0, 9);
	    $convertedPersianNums = str_replace($persian, $num, $string);
	    $englishNumbersOnly = str_replace($arabic, $num, $convertedPersianNums);

	    return $englishNumbersOnly;
	}

	function convert_to_fa_number($string) {
	    $num = range(0, 9);
	    $arabic = ['٩', '٨', '٧', '٦', '٥', '٤', '٣', '٢', '١','٠'];
	    $persian = ['۰', '۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹'];

	    $convertedEnglishNums = str_replace($num, $persian, $string);
	    $persianNumbersOnly = str_replace($arabic, $persian, $convertedEnglishNums);

	    return $persianNumbersOnly;
	}
?>